@@ -55,32 +55,38 @@ typedef struct TAG_MK_MEMORY_POOL
 
 // variable definition object
 struct TAG_MK_NODE_EXPR;
+struct TAG_MK_CLASS;
+struct TAG_MK_VM_STRUCT;
+typedef int (*FP_MK_NATIVE_METHOD)(struct TAG_MK_VM_STRUCT *vm, unsigned int varArgC );
 typedef struct TAG_MK_VARIABLE
 {
 	unsigned int flags;
 	MK_CHAR *name;
 	union
 	{
+		// for module variable
+		struct TAG_MK_CLASS *moduleVariables;
+
 		// for variable
 		struct TAG_MK_NODE_EXPR *defaultValue;
 		
 		// for method
 		struct
 		{
-			union
-			{
-				MK_VECTOR* args;				// script
-				unsigned int sizeArgs;			// native
-			};
-			union
-			{
-				struct TAG_MK_NODE_BLOCK *impl;	// script
-				void *entryPoint;				// native
-			};
+			MK_VECTOR* args;
+			void *entryPoint;	// script:MK_NODE_BLOCK* native:FP_MK_NATIVE_METHOD
 		};
 	};
 } MK_VARIABLE;
 
+// namespace object
+typedef struct TAG_MK_NAMESPACE
+{
+	unsigned int flags;
+	MK_CHAR *name;
+	MK_HASHTABLE *child;	// class, module, namespace
+} MK_NAMESPACE;
+
 // class code object
 typedef struct TAG_MK_CLASS
 {
@@ -88,6 +94,7 @@ typedef struct TAG_MK_CLASS
 	MK_CHAR *nameThis;
 	MK_CHAR *nameSuper;
 	MK_HASHTABLE* variables;
+	MK_VECTOR* usingNames;
 	INT_PTR *operatorMethod;
 } MK_CLASS;
 
@@ -101,22 +108,30 @@ typedef struct TAG_MK_NODE_EXPR
 
 	union
 	{
-		MK_CHAR *symbolName;				// method_call, symbol, new, delete
-		MK_CHAR *typeName;					// MK_VM_VARIABLE_INSTANCE
-		MK_VECTOR *multipleSymbols;			// multiple symbol top
-		MK_VECTOR *arrayDefinition;			// array definition.
-		unsigned int length;				// string
-		unsigned int constantValue;			// constant_value(high)
-		struct TAG_MK_NODE_EXPR *left;		// operation, return
-		unsigned int *node;					// node_with_param(MK_VARIABLE)
-	}u1;
-	union
-	{
-		struct TAG_MK_NODE_EXPR *right;		// operation
-		unsigned int constantValue;			// constant_value(low)
-		void *value;						// string
-		MK_VECTOR *args;					// method_call, new
-	}u2;
+		struct
+		{
+			union
+			{
+				MK_CHAR *symbolName;				// method_call, symbol, new, delete
+				MK_CHAR *typeName;					// MK_VM_VARIABLE_INSTANCE
+				MK_VECTOR *multipleSymbols;			// multiple symbol top
+				MK_VECTOR *arrayDefinition;			// array definition.
+				unsigned int length;				// string
+				unsigned int constantValue;			// constant_value(high)
+				struct TAG_MK_NODE_EXPR *left;		// operation, return
+				unsigned int *node;					// node_with_param(MK_VARIABLE)
+			}u1;
+			union
+			{
+				struct TAG_MK_NODE_EXPR *right;		// operation
+				unsigned int constantValue;			// constant_value(low)
+				void *value;						// string
+				MK_VECTOR *args;					// method_call, new
+			}u2;
+		};
+		MK_FLOAT floatValue;
+	};
+
 } MK_NODE_EXPR;
 
 // block( exprs ) code object
@@ -226,7 +241,18 @@ typedef struct TAG_MK_VM_FRAME_ITEM
 		} nativeObjectTypeValue;
 
 		// reference.
-		struct TAG_MK_VM_FRAME_ITEM *reference;
+		struct
+		{
+			MK_HASHTABLE *target;
+			MK_SYM_CHAR *symbolName;
+		} referenceTypeValue;
+
+		// reference array.
+		struct
+		{
+			MK_VECTOR *target;
+			INT32 index;
+		} referenceArrayType;
 	};
 } MK_VM_FRAME_ITEM;
 
@@ -330,15 +356,16 @@ extern const MK_KEYWORD_TABLE MK_RESERVED_MARK[];
 // --- vm cache data
 typedef struct TAG_MK_VM_CACHE
 {
-	MK_CLASS			*pConstantClass[0x0f];	// internal class
+	MK_SYM_CHAR			*internalClassSymbolName[0x0f];	// constant class symbolnae.
+	MK_CLASS			*pConstantClass[0x0f];			// constant class type.
 } MK_VM_CACHE;
 
 // ---- vm local stack
 typedef struct TAG_MK_VM_STACK
 {
-	unsigned int	**start;
-	unsigned int	**current;
-	unsigned int	**last;
+	MK_VM_FRAME_ITEM	**start;
+	MK_VM_FRAME_ITEM	**current;
+	MK_VM_FRAME_ITEM	**last;
 } MK_VM_STACK;
 
 // ---- vm core
@@ -350,8 +377,8 @@ typedef struct TAG_MK_VM_STRUCT
 	MK_MEMORY_POOL *memoryPool;			// memory pool
 	MK_HASHTABLE *hashSymbolName;		// symbol name => address of memory pool.
 
-	int (*callBlock[0x10])		(struct TAG_MK_VM_STRUCT *, unsigned int*);					// block call function table
-	int (*callNodeExpr[0xcf])	(struct TAG_MK_VM_STRUCT *, MK_NODE_EXPR*, int hasParent );	// expr call function table
+	int (*callBlock[0x10])		(struct TAG_MK_VM_STRUCT *, unsigned int*);									// block call function table
+	int (*callNodeExpr[0xcf])	(struct TAG_MK_VM_STRUCT *, MK_NODE_EXPR*, int hasParent, int reference );	// expr call function table
 
 	MK_OBJECTCODE *code;
 
@@ -379,20 +406,22 @@ typedef struct TAG_MK_VM_STRUCT
 // object type id
 #define MK_OBJECT_TYPE_MASK					0xf0000000
 #define MK_OBJECT_TYPE(value)				( (value) & MK_OBJECT_TYPE_MASK )
+#define MK_OBJECT_TYPE_INDEX(value)			( value >> 28 )
 #define MK_OBJECT_TYPE_VM_STRUCT			0x10000000		// MK_VM_STRUCT
 #define MK_OBJECT_TYPE_OBJECTCODE			0x20000000		// MK_OBJECTCODE
 #define MK_OBJECT_TYPE_VM_FRAME				0x30000000		// MK_VM_FRAME
-#define MK_OBJECT_TYPE_VM_FRAME_ITEM		0x40000000		// MK_VM_FRAME_ITEM(managed by gc)
-#define MK_TYPE_CLASS						0x50000000		// MK_CLASS
-#define MK_TYPE_VARIABLE					0x60000000		// MK_VARIABLE
-#define MK_TYPE_NODE_BLOCK					0x70000000		// MK_NODE_BLOCK
-#define MK_TYPE_NODE_IF						0x80000000		// MK_NODE_IF
-#define MK_TYPE_NODE_WHILE					0x90000000		// MK_NODE_WHILE
-#define MK_TYPE_TRY_BLOCK					0xa0000000		// MK_TRY_BLOCK
-#define MK_TYPE_CATCH_BLOCK					0xb0000000		// MK_CATCH_BLOCK
-#define MK_TYPE_NODE_EXPR					0xc0000000		// MK_NODE_EXPR
-#define MK_TYPE_HASHTABLE					0xd0000000		// MK_HASHTABLE
-#define MK_TYPE_VECTOR						0xe0000000		// MK_VECTOR
+#define MK_OBJECT_TYPE_VM_FRAME_ITEM		0x40000000		// MK_VM_FRAME_ITEM
+#define MK_TYPE_NAMESPACE					0x50000000		// MK_NAMESPACE
+#define MK_TYPE_CLASS						0x60000000		// MK_CLASS
+#define MK_TYPE_VARIABLE					0x70000000		// MK_VARIABLE
+#define MK_TYPE_NODE_BLOCK					0x80000000		// MK_NODE_BLOCK
+#define MK_TYPE_NODE_IF						0x90000000		// MK_NODE_IF
+#define MK_TYPE_NODE_WHILE					0xa0000000		// MK_NODE_WHILE
+#define MK_TYPE_TRY_BLOCK					0xb0000000		// MK_TRY_BLOCK
+#define MK_TYPE_CATCH_BLOCK					0xc0000000		// MK_CATCH_BLOCK
+#define MK_TYPE_NODE_EXPR					0xd0000000		// MK_NODE_EXPR
+#define MK_TYPE_HASHTABLE					0xe0000000		// MK_HASHTABLE
+#define MK_TYPE_VECTOR						0xf0000000		// MK_VECTOR
 
 // object attribute
 #define MK_TYPE_ATTRIBUTE_MASK					0x0fe00000
@@ -408,18 +437,20 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_VM_FRAME_TYPE_ROOT					0x00000000
 #define MK_VM_FRAME_TYPE_NODE_WITH_PARAM		0x01000000
 
+#define MK_VM_FRAME_ITEM_TYPE_GROUP(value)		( (value) & 0x0f000000 )
 // attribute of MK_OBJECT_TYPE_VM_FRAME_ITEM
 #define MK_VM_FRAME_ITEM_TYPE_NIL				0x00000000
 #define MK_VM_FRAME_ITEM_TYPE_CLASS				0x01000000
+#define MK_VM_FRAME_ITEM_TYPE_MODULE			0x01800000
 #define MK_VM_FRAME_ITEM_TYPE_INT_VALUE			0x02000000
 #define MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE		0x03000000
 #define MK_VM_FRAME_ITEM_TYPE_ARRAY_VALUE		0x04000000
 #define MK_VM_FRAME_ITEM_TYPE_STRING_VALUE		0x05000000
-#define MK_VM_FRAME_ITEM_TYPE_CHARACTER_VALUE	0x06000000
-#define MK_VM_FRAME_ITEM_TYPE_BOOL_VALUE		0x07000000
-#define MK_VM_FRAME_ITEM_TYPE_NATIVE_PTR		0x08000000
-#define MK_VM_FRAME_ITEM_TYPE_REFERENCE_VALUE	0x09000000
-#define MK_VM_FRAME_ITEM_TYPE_NODE_VALUE		0x0a000000
+#define MK_VM_FRAME_ITEM_TYPE_NODE_VALUE		0x06000000
+#define MK_VM_FRAME_ITEM_TYPE_REFERENCE_VALUE	0x0d000000
+#define MK_VM_FRAME_ITEM_TYPE_REFERENCE_ARRAY_VALUE			\
+												0x0d800000
+#define MK_VM_FRAME_ITEM_TYPE_NATIVE_PTR		0x0e000000
 #define MK_VM_FRAME_ITEM_TYPE_THIS				0x0f200000
 #define MK_VM_FRAME_ITEM_TYPE_SUPER				0x0f400000
 #define MK_VM_FRAME_ITEM_TYPE_OWNER				0x0f600000
@@ -440,20 +471,21 @@ typedef struct TAG_MK_VM_STRUCT
 // MK_VM_FRAME_ITEM_PTR value
 #define MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE		0x00000001
 #define MK_VM_FRAME_ITEM_TYPE_DIRECT_MASK		0x70000000
-#define MK_VM_FRAME_ITEM_TYPE_DIRECT_INT		0x00000000
-#define MK_VM_FRAME_ITEM_TYPE_DIRECT_BOOL		0x10000000
-#define MK_VM_FRAME_ITEM_TYPE_DIRECT_CHAR		0x20000000
 #define MK_VM_FRAME_ITEM_TYPE_INT_MINUS_VALUE	0x80000000
-#define MK_VM_FRAME_ITEM_DIRECT_INT_VALUE_MASK	0x0ffffff0	// -16777215 ~ +16777215
+#define MK_VM_FRAME_ITEM_DIRECT_INT_VALUE_MASK	0x7ffffffe	// -1,073,741,823 ~ +1,073,741,823
 
 // attribute of MK_TYPE_CLASS
-
-// attribute of MK_TYPE_FUNCTION
+#define MK_TYPE_ATTRIBUTE_CLASS_CLASS				0x00000000
+#define MK_TYPE_ATTRIBUTE_CLASS_MODULE				0x01000000
+#define MK_TYPE_ATTRIBUTE_CLASS_TYPE_INTERNAL		0x00200000
 
 // attribute of MK_TYPE_VARIABLE
 #define MK_TYPE_ATTRIBUTE_VARIABLE_VALUE			0x00000000	// variable type
 #define MK_TYPE_ATTRIBUTE_VARIABLE_METHOD			0x08000000	// metod type
 
+// style only variable
+#define MK_TYPE_ATTRIBUTE_VARIABLE_MODULE			0x04000000	// module instance
+
 // style only method
 #define MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_SCRIPT	0x00000000
 #define MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE	0x04000000
@@ -481,6 +513,9 @@ typedef struct TAG_MK_VM_STRUCT
 // method arguments param(testing)
 #define MK_TYPE_ATTRIBUTE_VARIABLE_CALL_BY_VALUE	0x00000800
 
+// native method new style(tesiting)
+#define MK_TYPE_ATTRIBUTE_VARIABLE_NEW_NATIVE_METHOD	0x00000010
+
 // attribute of MK_TYPE_BLOCK
 #define MK_TYPE_ATTRIBUTE_BLOCK_FUNCTION_ROOT		0x00000000
 #define MK_TYPE_ATTRIBUTE_BLOCK_IF_BLOCK			0x01000000
@@ -518,18 +553,16 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_TYPE_NODE_EXPR_NEW				0x0a000000
 #define MK_TYPE_NODE_EXPR_RAISE				0x0b000000
 #define MK_TYPE_NODE_EXPR_NODE_WITH_PARAM	0x0c000000
-
-#define MK_TYPE_NODE_EXPR_ARRAY_DEFINITION	0x0d000000
+#define MK_TYPE_NODE_EXPR_BACK_IF_CONDITION	0x0d200000
+#define MK_TYPE_NODE_EXPR_ARRAY_DEFINITION	0x0e000000
+#define MK_TYPE_NODE_EXPR_ME				0x0e200000
 
 #define MK_TYPE_NODE_EXPR_CONSTANT		0x00e00000
-#define MK_TYPE_NODE_EXPR_INT8			0x03e00000
-#define MK_TYPE_NODE_EXPR_INT16			0x04e00000
 #define MK_TYPE_NODE_EXPR_INT32			0x05e00000
 #define MK_TYPE_NODE_EXPR_INT64			0x06e00000
 #define MK_TYPE_NODE_EXPR_FLOAT			0x07e00000
 #define MK_TYPE_NODE_EXPR_STRING		0x08e00000
-#define MK_TYPE_NODE_EXPR_CHAR			0x09e00000
-#define MK_TYPE_NODE_EXPR_CONST_FINAL	0x00100000	// never change value
+#define MK_TYPE_NODE_EXPR_CONST_FINAL	0x00100000	// never changed value
 
 #define MK_TYPE_NODE_EXPR_OPERATIONTYPE_MASK		0x000fc000
 #define MK_TYPE_NODE_EXPR_OPERATIONPRIORITY_MASK	0x00003c00
@@ -573,26 +606,27 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_LEX_TYPE_RESERVED_MARK_SMALL		MK_MAKE_LEX_RESERVED_MARK( 0x10 - 7, 17, 1, 1 )
 #define MK_LEX_TYPE_RESERVED_MARK_EQUAL		MK_MAKE_LEX_RESERVED_MARK( 0x10 - 8, 18, 0, 1 )
 #define MK_LEX_TYPE_RESERVED_MARK_DOT		MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 19, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_PARENTHESIS		MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 20, 0, 0 )	// (
-#define MK_LEX_TYPE_RESERVED_MARK_END_PARENTHESIS	MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 21, 0, 0 )	// )
-#define MK_LEX_TYPE_RESERVED_MARK_BRACKET			MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 22, 0, 0 )	// [
-#define MK_LEX_TYPE_RESERVED_MARK_END_BRACKET		MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 23, 0, 0 )	// ]
-#define MK_LEX_TYPE_RESERVED_MARK_BRACE				MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 24, 0, 0 )	// {
-#define MK_LEX_TYPE_RESERVED_MARK_END_BRACE			MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1, 25, 0, 0 )	// }
+#define MK_LEX_TYPE_RESERVED_MARK_PARENTHESIS		MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1,	20, 0, 0 )	// (
+#define MK_LEX_TYPE_RESERVED_MARK_END_PARENTHESIS	MK_MAKE_LEX_RESERVED_MARK( 0,			21, 0, 0 )	// )
+#define MK_LEX_TYPE_RESERVED_MARK_BRACKET_REF		MK_MAKE_LEX_RESERVED_MARK( 0,			22, 0, 0 )  // [ reference.
+#define MK_LEX_TYPE_RESERVED_MARK_BRACKET			MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1,	23, 0, 0 )	// [
+#define MK_LEX_TYPE_RESERVED_MARK_END_BRACKET		MK_MAKE_LEX_RESERVED_MARK( 0,			24, 0, 0 )	// ]
+#define MK_LEX_TYPE_RESERVED_MARK_BRACE				MK_MAKE_LEX_RESERVED_MARK( 0x10 - 1,	25, 0, 0 )	// {
+#define MK_LEX_TYPE_RESERVED_MARK_END_BRACE			MK_MAKE_LEX_RESERVED_MARK( 0,			26, 0, 0 )	// }
 
-#define MK_LEX_TYPE_RESERVED_MARK_DBLATMARK		MK_MAKE_LEX_RESERVED_MARK( 0, 26, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_ATMARK		MK_MAKE_LEX_RESERVED_MARK( 0, 27, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_DBLATMARK		MK_MAKE_LEX_RESERVED_MARK( 0, 27, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_ATMARK		MK_MAKE_LEX_RESERVED_MARK( 0, 28, 0, 0 )
 
-#define MK_LEX_TYPE_RESERVED_MARK_EXCLAMATION	MK_MAKE_LEX_RESERVED_MARK( 0, 28, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_QUESTION		MK_MAKE_LEX_RESERVED_MARK( 0, 29, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_CORON			MK_MAKE_LEX_RESERVED_MARK( 0, 30, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_DOUBLE_QUATE	MK_MAKE_LEX_RESERVED_MARK( 0, 31, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_SINGLE_QUATE	MK_MAKE_LEX_RESERVED_MARK( 0, 32, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_CONMA			MK_MAKE_LEX_RESERVED_MARK( 0, 33, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_CHILDA		MK_MAKE_LEX_RESERVED_MARK( 0, 34, 0, 0 )
-#define MK_LEX_TYPE_RESERVED_MARK_SHARP			MK_MAKE_LEX_RESERVED_MARK( 0, 35, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_EXCLAMATION	MK_MAKE_LEX_RESERVED_MARK( 0, 29, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_QUESTION		MK_MAKE_LEX_RESERVED_MARK( 0, 30, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_CORON			MK_MAKE_LEX_RESERVED_MARK( 0, 31, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_DOUBLE_QUATE	MK_MAKE_LEX_RESERVED_MARK( 0, 32, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_SINGLE_QUATE	MK_MAKE_LEX_RESERVED_MARK( 0, 33, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_CONMA			MK_MAKE_LEX_RESERVED_MARK( 0, 34, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_CHILDA		MK_MAKE_LEX_RESERVED_MARK( 0, 35, 0, 0 )
+#define MK_LEX_TYPE_RESERVED_MARK_SHARP			MK_MAKE_LEX_RESERVED_MARK( 0, 36, 0, 0 )
 
-#define MK_SIZEOF_ENABELD_OPELATORS	26
+#define MK_SIZEOF_ENABELD_OPELATORS	27
 
 // hashtable
 #define MK_TYPE_HASHTABLE_SIZE_EXTEND_MASK	0x000ffc00	// 0 ~ 1023
@@ -689,7 +723,7 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_LEX_TYPE_RESERVED_SYMBOL_TRUE	MK_LEX_TYPE_RESERVED_SYMBOL + 17
 #define MK_LEX_TYPE_RESERVED_SYMBOL_FALSE	MK_LEX_TYPE_RESERVED_SYMBOL + 18
 #define MK_LEX_TYPE_RESERVED_SYMBOL_RAISE	MK_LEX_TYPE_RESERVED_SYMBOL + 19
-#define MK_LEX_TYPE_RESERVED_SYMBOL_TRY	MK_LEX_TYPE_RESERVED_SYMBOL + 20
+#define MK_LEX_TYPE_RESERVED_SYMBOL_TRY		MK_LEX_TYPE_RESERVED_SYMBOL + 20
 #define MK_LEX_TYPE_RESERVED_SYMBOL_CATCH	MK_LEX_TYPE_RESERVED_SYMBOL + 21
 #define MK_LEX_TYPE_RESERVED_SYMBOL_FINALLY	MK_LEX_TYPE_RESERVED_SYMBOL + 22
 #define MK_LEX_TYPE_RESERVED_SYMBOL_PUBLIC	MK_LEX_TYPE_RESERVED_SYMBOL + 23
@@ -699,6 +733,11 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_LEX_TYPE_RESERVED_SYMBOL_WRITE		MK_LEX_TYPE_RESERVED_SYMBOL + 27
 #define MK_LEX_TYPE_RESERVED_SYMBOL_INITIALIZE	MK_LEX_TYPE_RESERVED_SYMBOL + 28
 #define MK_LEX_TYPE_RESERVED_SYMBOL_BLOCK		MK_LEX_TYPE_RESERVED_SYMBOL + 29
+#define MK_LEX_TYPE_RESERVED_SYMBOL_NAMESPACE	MK_LEX_TYPE_RESERVED_SYMBOL + 30
+#define MK_LEX_TYPE_RESERVED_SYMBOL_IMPORT		MK_LEX_TYPE_RESERVED_SYMBOL + 31
+#define MK_LEX_TYPE_RESERVED_SYMBOL_USING		MK_LEX_TYPE_RESERVED_SYMBOL + 32
+#define MK_LEX_TYPE_RESERVED_SYMBOL_MODULE		MK_LEX_TYPE_RESERVED_SYMBOL + 33
+#define MK_LEX_TYPE_RESERVED_SYMBOL_ME			MK_LEX_TYPE_RESERVED_SYMBOL + 34
 
 #define MK_LEX_STRING_MARK					'\"'
 #define MK_LEX_CHARACTER_MARK				'\''
@@ -749,6 +788,7 @@ int mk_find_item_hashtable( MK_HASHTABLE *ptable, const MK_CHAR *key, void **val
 int mk_is_key_hashtable( MK_HASHTABLE *table, const MK_CHAR *key );
 void mk_destroy_hashtable_node( MK_MEMORY_POOL *top, MK_HASHTABLE *target, int isDeleteValue );
 MK_CHAR *mk_get_symbol_name_ptr( MK_VM_STRUCT *vm, const MK_CHAR *target );
+int mk_remove_key_hashtable( MK_HASHTABLE *table, const MK_CHAR *key, void **removed );
 
 void* mk_enum_item_hashtable_begin( MK_HASHTABLE *ptable );
 void* mk_enum_item_hashtable_next( MK_HASHTABLE *ptable, void *iterator, const MK_CHAR **key, void **value );
@@ -772,54 +812,52 @@ void mk_clear_lexer( MK_LEXER *lexer );
 int mk_get_token( MK_LEXER *lexer );
 
 // compile
-int do_compile( MK_VM_STRUCT *vm, void*target );
+int mk_do_compile( MK_VM_STRUCT *vm, void*target );
 
 // dynamic code generate
 int mk_register_class( MK_VM_STRUCT *vm, MK_CLASS *target );
 int mk_register_variable( MK_VM_STRUCT *vm, MK_VARIABLE *targetVariable, MK_CLASS *targetClass );
 int mk_register_variable_arg( MK_VM_STRUCT *vm, MK_VECTOR *targetArgs, MK_VARIABLE *arg );
 MK_VARIABLE *mk_create_variable( MK_VM_STRUCT *vm, MK_CHAR *mk_name, unsigned int type, MK_NODE_EXPR *defaultValue );
-MK_VARIABLE *mk_create_method( MK_VM_STRUCT *vm, MK_CHAR *name, unsigned int type, INT_PTR arg, INT_PTR impl );
-
+MK_VARIABLE *mk_create_method( MK_VM_STRUCT *vm, MK_CHAR *name, unsigned int type, MK_VECTOR* arg, INT_PTR entryPoint );
+MK_VARIABLE *mk_create_default_native_method( MK_VM_STRUCT *vm, MK_CHAR *name, unsigned int type, int sizeArg, INT_PTR entryPoint );
 // dump object(for debug)
 void dump_node( void *target, unsigned int rank );
 
 // vm
+int mk_vm_initialize_object_variables( MK_VM_STRUCT *vm, MK_CLASS *pClass, MK_VM_FRAME_ITEM *target, int isStatic );
 int mk_vm_execute_expr_default_value( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *pOwner, MK_VARIABLE *valueVariable, MK_VM_FRAME_ITEM *defaultValue );
 int mk_vm_initialize( MK_VM_STRUCT *vm );
 void mk_register_internal_classes( MK_VM_STRUCT *vm );
 int mk_vm_run( MK_VM_STRUCT *vm );
 int mk_execute_block( MK_VM_STRUCT *vm, unsigned int *target );
-unsigned int mk_native_method_call( MK_VM_STRUCT *vm, MK_VARIABLE *pMethod, unsigned int sizeParam );
-int mk_vm_frame_item_to_int32( MK_VM_FRAME_ITEM *target );
-MK_FLOAT mk_vm_frame_item_to_float( MK_VM_FRAME_ITEM *target );
+int mk_vm_frame_item_to_int32( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target );
+MK_FLOAT mk_vm_frame_item_to_float( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target );
 MK_VM_FRAME_ITEM *mk_vm_create_int32_frame_item( MK_VM_STRUCT *vm, int value );
 MK_VM_FRAME_ITEM *mk_vm_create_float_frame_item( MK_VM_STRUCT *vm, MK_FLOAT value );
 MK_VM_FRAME_ITEM *mk_vm_create_bool_frame_item( int isTrue );
 int mk_vm_frame_item_is_true( MK_VM_FRAME_ITEM *target );
+MK_VM_FRAME_ITEM *mk_vm_find_instance( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *instance, MK_SYM_CHAR *className );
 
 void mk_destroy_vm_frame_item( MK_MEMORY_POOL *top, MK_VM_FRAME_ITEM *target );
 
 #include "vm_stack.h"
 
-#define mk_execute_expr( vm, pTarget, hasParent )														\
-( ( abs( (INT_PTR)((vm)->stackTop) - (INT_PTR)&(vm) ) < MK_SIZEOF_MAX_STACK_SIZE ) ?					\
-		vm->callNodeExpr[MK_TYPE_ATTRIBUTE_INDEX((pTarget)->flags)]((vm), (pTarget), (hasParent) ) :	\
+#define mk_execute_expr( vm, pTarget, hasParent, reference )														\
+( ( abs( (INT_PTR)((vm)->stackTop) - (INT_PTR)&(vm) ) < MK_SIZEOF_MAX_STACK_SIZE ) ?								\
+		vm->callNodeExpr[MK_TYPE_ATTRIBUTE_INDEX((pTarget)->flags)]((vm), (pTarget), (hasParent), (reference ) ) :	\
 		mk_raise_internal_error( vm, "", 0, MK_ERROR_TYPE_VM_ERROR | 9, NULL ) )
 MK_CLASS *mk_vm_get_class_by_name( MK_VM_STRUCT *vm, const MK_CHAR *pname );
 MK_CLASS *mk_vm_get_class( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *pTarget );
 int mk_vm_call_method( MK_VM_STRUCT *vm, MK_VARIABLE *pMethod, MK_VM_FRAME *pOwnerFrame );
 
-#define mk_get_entity_vm_frame_item( item )			\
-{													\
-	while( MK_TYPE_ATTRIBUTE(item->flags) ==		\
-		MK_VM_FRAME_ITEM_TYPE_REFERENCE_VALUE )		\
-		item = item->reference;						\
-}
-
 // special class method
 int mk_object_new( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM **result );
 int mk_object_equal( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target, MK_VM_FRAME_ITEM *right, MK_VM_FRAME_ITEM **result );
+MK_VM_FRAME_ITEM *mk_reference_to_value( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *source );
+
+int mk_node_invoke( struct TAG_MK_VM_STRUCT *vm, unsigned int varArgC );
+
 
 // object create/destroy
 void *mk_create_object( unsigned int nsType );
@@ -858,31 +896,36 @@ INT_PTR mk_get_at_vector( MK_VECTOR *ptarget, unsigned int index );
 const MK_CHAR *symbol_int_to_string( unsigned int value );
 const MK_CHAR *operation_int_to_string( unsigned int value );
 MK_VARIABLE* mk_vm_find_method( MK_VM_STRUCT *vm, 
-								 MK_VM_FRAME_ITEM *pOwner, const MK_CHAR *name, 
+								 MK_VM_FRAME_ITEM *pOwner, const MK_SYM_CHAR *name, 
 								 unsigned int sizeArgs );
 MK_VM_FRAME_ITEM* mk_vm_find_variable( MK_VM_STRUCT *vm, 
-								 MK_VM_FRAME_ITEM *pOwner, const MK_CHAR *name );
+								 MK_VM_FRAME_ITEM *pOwner, const MK_SYM_CHAR *name );
+MK_VM_FRAME_ITEM* mk_vm_find_variable_reference( MK_VM_STRUCT *vm, 
+								 MK_VM_FRAME_ITEM *pOwner, const MK_SYM_CHAR *name );
 
 // gc
 int mk_gc_run( MK_VM_STRUCT *vm, int force );
 
-#define CLASS_INTERNAL_OBJECT	"Object"
-#define CLASS_INTERNAL_NULL		"Null"
+#define CLASS_INTERNAL_OBJECT		"Object"
+#define CLASS_INTERNAL_NULL			"Null"
 
-#define CLASS_INTERNAL_KERNEL "Kernel"
+#define CLASS_INTERNAL_REFERENCE	"Reference"
+#define CLASS_INTERNAL_KERNEL		"Kernel"
 
-#define CLASS_INTERNAL_INTEGER	"Integer"
-#define	CLASS_INTERNAL_FLOAT	"Float"
+#define CLASS_INTERNAL_STRING		"String"
 
-#define CLASS_INTERNAL_IO		"IO"
-#define CLASS_INTERNAL_CONSOLE	"Console"
+#define CLASS_INTERNAL_INTEGER		"Integer"
+#define	CLASS_INTERNAL_FLOAT		"Float"
+
+#define CLASS_INTERNAL_IO			"IO"
+#define CLASS_INTERNAL_CONSOLE		"Console"
 
 #define CLASS_INTERNAL_CONTAINER	"Container"
-#define CLASS_INTERNAL_ARRAY	"Array"
+#define CLASS_INTERNAL_ARRAY		"Array"
 
-#define CLASS_INTERNAL_NODE		"Node"
+#define CLASS_INTERNAL_NODE			"Node"
 
-#define CLASS_INTERNAL_EXCEPTION "Exception"
+#define CLASS_INTERNAL_EXCEPTION	"Exception"
 #define CLASS_INTERNAL_COMPILE_ERROR_EXCEPTION	"CompileErrorException"
 #define CLASS_INTERNAL_VM_ERROR_EXCEPTION	"VmErrorException"
 #define CLASS_INTERNAL_ARRAY_OUTOF_RANGE	"ArrayOutOfRangeException"
