@@ -4,20 +4,20 @@
 #include <string.h>
 #include <stdio.h>
 
-#ifdef _DEBUG
-static unsigned int magicNumber = 0;
-#endif
-
 static
 void mk_extend_hashtable( MK_HASHTABLE *ptable );
 static
 int mk_find_item_position_string( MK_HASHTABLE *table, const MK_CHAR *key, void **result );
 static
-int mk_seek_insert_position_string( MK_HASHTABLE *table, const MK_CHAR *key );
+int mk_seek_insert_position_string( MK_HASHTABLE *table, const MK_CHAR *key, int *isKey );
 static
 int mk_find_item_position_ptr( MK_HASHTABLE *table, const INT_PTR key, void **result );
 static
-int mk_seek_insert_position_ptr( MK_HASHTABLE *table, const INT_PTR key);
+int mk_seek_insert_position_ptr( MK_HASHTABLE *table, const INT_PTR key, int *isKey );
+static
+int mk_remove_key_hashtable_string( MK_HASHTABLE *table, const MK_CHAR *key, void **removed );
+static
+int mk_remove_key_hashtable_intptr( MK_HASHTABLE *table, const INT_PTR key, void **removed );
 
 MK_HASHTABLE *mk_create_hashtable( unsigned int defaultSize, unsigned int extend )
 {
@@ -28,7 +28,7 @@ MK_HASHTABLE *mk_create_hashtable( unsigned int defaultSize, unsigned int extend
 
 	result->size = defaultSize;
 	result->used = 0;
-	result->flags = MK_TYPE_HASHTABLE | ( ( extend << 18 ) & MK_TYPE_HASHTABLE_SIZE_EXTEND_MASK );
+	result->flags = MK_TYPE_HASHTABLE | ( ( extend << 10 ) & MK_TYPE_HASHTABLE_SIZE_EXTEND_MASK );
 	result->flags |= MK_TYPE_ATTRIBUTE_HASHKEY_INTPTR;
 	result->elems = malloc( sizeElem );
 	return result;
@@ -79,7 +79,7 @@ MK_HASHTABLE *mk_allocate_vm_managed_hashtable( MK_MANAGED_HASH_TABLE **managedH
 	}
 
 	pCurrent->flags = 
-		MK_TYPE_HASHTABLE | ( ( MK_SIZEOF_EXTEND_HASH_DEFAULT << 18 ) & MK_TYPE_HASHTABLE_SIZE_EXTEND_MASK );
+		MK_TYPE_HASHTABLE | ( ( MK_SIZEOF_EXTEND_HASH_DEFAULT << 10 ) & MK_TYPE_HASHTABLE_SIZE_EXTEND_MASK );
 	MK_GC_SET_MANAGED_BIT( &pCurrent->flags );
 	MK_GC_ALLOCATE_OBJECT( &pCurrent->flags );
 	MK_GC_SET_CN_BIT( &pCurrent->flags );
@@ -89,6 +89,7 @@ MK_HASHTABLE *mk_allocate_vm_managed_hashtable( MK_MANAGED_HASH_TABLE **managedH
 	{
 		pCurrent->elems = 
 			malloc( sizeof( MK_MAP_ITEM ) * MK_SIZEOF_HASH_DEFAULT );
+		
 		pCurrent->size = MK_SIZEOF_HASH_DEFAULT;
 	}
 	pCurrent->used = 0;
@@ -99,24 +100,26 @@ MK_HASHTABLE *mk_allocate_vm_managed_hashtable( MK_MANAGED_HASH_TABLE **managedH
 
 unsigned int mk_insert_item_hashtable( MK_HASHTABLE *table, const MK_CHAR *key, void *value )
 {
-	int insertIndex = 0;
-
+	unsigned int insertIndex = 0;
+	int isKey = 0;
 	if( table->used > 0 )
 	{
 		switch( MK_TYPE_ATTRIBUTE_HASHKEY_STYLE( table->flags ) )
 		{
 		case MK_TYPE_ATTRIBUTE_HASHKEY_STRING:
-			insertIndex = mk_seek_insert_position_string( table, key );
+			insertIndex = mk_seek_insert_position_string( table, key, &isKey );
 			break;
 		case MK_TYPE_ATTRIBUTE_HASHKEY_INTPTR:
-			insertIndex = mk_seek_insert_position_ptr( table, (INT_PTR)key );
-			break;
-		case MK_TYPE_ATTRIBUTE_HASHKEY_INT32:
-		case MK_TYPE_ATTRIBUTE_HASHKEY_INT64:
+			insertIndex = mk_seek_insert_position_ptr( table, (INT_PTR)key, &isKey );
 			break;
 		}
 	}
-	if( insertIndex >= 0 )
+
+	if( isKey != 0 )
+	{
+		table->elems[insertIndex].value = value;
+	}
+	else
 	{
 		if( table->size == table->used )
 			mk_extend_hashtable( table );
@@ -133,15 +136,11 @@ unsigned int mk_insert_item_hashtable( MK_HASHTABLE *table, const MK_CHAR *key,
 		case MK_TYPE_ATTRIBUTE_HASHKEY_INTPTR:
 			table->elems[insertIndex].intPtrKey = (INT_PTR)key;
 			break;
-		case MK_TYPE_ATTRIBUTE_HASHKEY_INT32:
-		case MK_TYPE_ATTRIBUTE_HASHKEY_INT64:
-			break;
 		}
 		table->elems[insertIndex].value = value;
 		table->used ++;
-		return insertIndex;
 	}
-	return (INT_PTR)insertIndex;
+	return insertIndex;
 }
 
 int mk_is_key_hashtable( MK_HASHTABLE *table, const MK_CHAR *key )
@@ -155,15 +154,59 @@ int mk_is_key_hashtable( MK_HASHTABLE *table, const MK_CHAR *key )
 		ret = mk_find_item_position_string( table, key, &value );
 		break;
 	case MK_TYPE_ATTRIBUTE_HASHKEY_INTPTR:
-		ret = mk_find_item_position_ptr( table, (const int)key, &value );
-		break;
-	case MK_TYPE_ATTRIBUTE_HASHKEY_INT32:
-	case MK_TYPE_ATTRIBUTE_HASHKEY_INT64:
+		ret = mk_find_item_position_ptr( table, (const INT_PTR)key, &value );
 		break;
 	}
 	return ret >= 0;
 }
 
+static
+int mk_remove_key_hashtable_string( MK_HASHTABLE *table, const MK_CHAR *key, void **removed )
+{
+ 	int index = mk_find_item_position_string( table, key, removed );
+	if( index >= 0 )
+	{
+		*removed = table->elems[index].value;
+		memmove( 
+			&table->elems[index], 
+			&table->elems[index+1], 
+			sizeof( MK_MAP_ITEM ) * ( table->used - index - 1 ) );
+		table->used --;
+	}
+	return 0;
+}
+
+static
+int mk_remove_key_hashtable_intptr( MK_HASHTABLE *table, const INT_PTR key, void **removed )
+{
+	int index = mk_find_item_position_ptr( table, key, removed );
+	if( index >= 0 )
+	{
+		*removed = table->elems[index].value;
+		memmove( 
+			&table->elems[index], 
+			&table->elems[index+1], 
+			sizeof( MK_MAP_ITEM ) * ( table->used - index - 1 ) );
+		table->used --;
+	}
+	return 0;
+}
+
+int mk_remove_key_hashtable( MK_HASHTABLE *table, const MK_CHAR *key, void **removed )
+{
+	int ret = 0;
+	switch( MK_TYPE_ATTRIBUTE_HASHKEY_STYLE( table->flags ) )
+	{
+	case MK_TYPE_ATTRIBUTE_HASHKEY_STRING:
+		ret = mk_remove_key_hashtable_string( table, key, removed );
+		break;
+	case MK_TYPE_ATTRIBUTE_HASHKEY_INTPTR:
+		ret = mk_remove_key_hashtable_intptr( table, (const INT_PTR)key, removed );
+		break;
+	}
+	return ret;
+}
+
 int mk_find_item_hashtable( MK_HASHTABLE *ptable, const MK_CHAR *key, void **value )
 {
 	int ret = 0;
@@ -176,9 +219,6 @@ int mk_find_item_hashtable( MK_HASHTABLE *ptable, const MK_CHAR *key, void **val
 	case MK_TYPE_ATTRIBUTE_HASHKEY_STRING:
 		ret = mk_find_item_position_string( ptable, key, value );
 		break;
-	case MK_TYPE_ATTRIBUTE_HASHKEY_INT32:
-	case MK_TYPE_ATTRIBUTE_HASHKEY_INT64:
-		break;	// todo: not supported yet.
 	}
 	if( ret < 0 )
 		*value = NULL;
@@ -273,7 +313,7 @@ int mk_find_item_position_string( MK_HASHTABLE *table, const MK_CHAR *key, void
 }
 
 static
-int mk_seek_insert_position_ptr( MK_HASHTABLE *table, const INT_PTR key )
+int mk_seek_insert_position_ptr( MK_HASHTABLE *table, const INT_PTR key, int *isKey )
 {
 	int high = table->used - 1, 
 		low = 0, 
@@ -287,17 +327,25 @@ int mk_seek_insert_position_ptr( MK_HASHTABLE *table, const INT_PTR key )
 		mid = ( low + high ) / 2;
 		target = elems[mid].intPtrKey;
 		if( target == key )
+		{
+			*isKey = 1;
 			return mid;
+		}
 		else if(target < key )
+		{
 			low = mid + 1;
+		}
 		else
+		{
 			high = mid - 1;
+		}
 	}
+	*isKey = 0;
 	return low;
 }
 
 static
-int mk_seek_insert_position_string( MK_HASHTABLE *table, const MK_CHAR *key )
+int mk_seek_insert_position_string( MK_HASHTABLE *table, const MK_CHAR *key, int *isKey )
 {
 	int high = table->used - 1, 
 		low = 0, 
@@ -311,13 +359,17 @@ int mk_seek_insert_position_string( MK_HASHTABLE *table, const MK_CHAR *key )
 		mid = ( low + high ) / 2;
 		ret = strcmp( elems[mid].key, key );
 		if( ret == 0 )
-			return mid;
+		{
+			*isKey = 1;
+			return mid;		// key already registerd.
+		}
 		else if( ret < 0 )
 			low = mid + 1;
 		else
 			high = mid - 1;
 	}
-	return low;
+	*isKey = 0;
+	return low;	// key not found.
 }
 
 static
