@@ -3,9 +3,11 @@
 
 void mk_trace( unsigned int code, MK_CHAR *message )
 {
-	fprintf( stdout, "[%08x]", code );
-	fputs( message, stdout );
-	fputs( "\n", stdout );
+	FILE *out = ( (code & MK_TRACE_TYPE_MASK) & MK_TRACE_TYPE_ERROR ) ? stderr : stdout;
+
+	fprintf( out, "[%08x]", code );
+	fputs( message, out );
+	fputs( "\n", out );
 }
 
 void *mk_open( void*value )
