@@ -252,8 +252,8 @@ MK_VM_FRAME_ITEM *mk_vm_create_bool_frame_item( int isTrue )
 {
 	return ( MK_VM_FRAME_ITEM * )
 		( ( isTrue ) ? 
-			MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE | MK_VM_FRAME_ITEM_TYPE_DIRECT_BOOL | 0x00000010:
-			MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE | MK_VM_FRAME_ITEM_TYPE_DIRECT_BOOL );
+			MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE | 0x00000002 : 
+			MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE | 0x00000000 );
 }
 
 int mk_vm_frame_item_is_true( MK_VM_FRAME_ITEM *target )
@@ -270,7 +270,6 @@ int mk_vm_frame_item_is_true( MK_VM_FRAME_ITEM *target )
 			return 0;
 
 		case MK_VM_FRAME_ITEM_TYPE_INT_VALUE:
-		case MK_VM_FRAME_ITEM_TYPE_BOOL_VALUE:
 			return target->int32TypeValue != 0;
 
 		case MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE:
@@ -295,11 +294,11 @@ MK_VM_FRAME_ITEM *mk_vm_create_float_frame_item( MK_VM_STRUCT *vm, MK_FLOAT valu
 MK_VM_FRAME_ITEM *mk_vm_create_int32_frame_item( MK_VM_STRUCT *vm, int value )
 {
 	MK_VM_FRAME_ITEM *result = NULL;
-	if( abs( value ) < 0x01000000 )
+	if( abs( value ) < 0x4000000 )
 	{
 		unsigned int intResult = 0;
 		intResult = 
-			MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE | MK_VM_FRAME_ITEM_TYPE_DIRECT_INT | ( abs( value ) << 4 );
+			MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE | ( abs( value ) << 1 );
 		if( value < 0 )
 			intResult |= MK_VM_FRAME_ITEM_TYPE_INT_MINUS_VALUE;
 		result = ( MK_VM_FRAME_ITEM* )intResult;
@@ -314,8 +313,9 @@ MK_VM_FRAME_ITEM *mk_vm_create_int32_frame_item( MK_VM_STRUCT *vm, int value )
 	return result;
 }
 
-MK_FLOAT mk_vm_frame_item_to_float( MK_VM_FRAME_ITEM *target )
+MK_FLOAT mk_vm_frame_item_to_float( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target )
 {
+	MK_VM_FRAME_ITEM *realTarget = NULL;
 	MK_FLOAT num = 0.0;
 	if( (INT_PTR)target & MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE )
 	{
@@ -334,17 +334,30 @@ MK_FLOAT mk_vm_frame_item_to_float( MK_VM_FRAME_ITEM *target )
 		case MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE:
 			num = target->floatTypeValue;
 			break;
+		case MK_VM_FRAME_ITEM_TYPE_CLASS:
+		case MK_VM_FRAME_ITEM_TYPE_MODULE:
+			realTarget = 
+				mk_vm_find_instance( vm, 
+					target, 
+					vm->cache->internalClassSymbolName[MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE >> 24] );
+			if( realTarget == NULL )
+				realTarget = 
+					mk_vm_find_instance( vm, 
+						target, 
+						vm->cache->internalClassSymbolName[MK_VM_FRAME_ITEM_TYPE_INT_VALUE >> 24] );
+			num = mk_vm_frame_item_to_float( vm, realTarget );
 		}
 	}
 	return num;
 }
 
-int mk_vm_frame_item_to_int32( MK_VM_FRAME_ITEM *target )
+int mk_vm_frame_item_to_int32( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *target )
 {
+	MK_VM_FRAME_ITEM *realTarget = NULL;
 	int result = 0;
 	if( (INT_PTR)target & MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE )
 	{
-		result = ( (INT_PTR)target & MK_VM_FRAME_ITEM_DIRECT_INT_VALUE_MASK ) >> 4;
+		result = ( (INT_PTR)target & MK_VM_FRAME_ITEM_DIRECT_INT_VALUE_MASK ) >> 1;
 		if( (INT_PTR)target & MK_VM_FRAME_ITEM_TYPE_INT_MINUS_VALUE )
 			result *= -1;
 	}
@@ -359,6 +372,18 @@ int mk_vm_frame_item_to_int32( MK_VM_FRAME_ITEM *target )
 		case MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE:
 			result = (int)target->floatTypeValue;
 			break;
+		case MK_VM_FRAME_ITEM_TYPE_CLASS:
+		case MK_VM_FRAME_ITEM_TYPE_MODULE:
+			realTarget = 
+				mk_vm_find_instance( vm, 
+					target, 
+					vm->cache->internalClassSymbolName[MK_VM_FRAME_ITEM_TYPE_INT_VALUE >> 24] );
+			if( realTarget == NULL )
+				realTarget = 
+					mk_vm_find_instance( vm, 
+						target, 
+						vm->cache->internalClassSymbolName[MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE >> 24] );
+			result = mk_vm_frame_item_to_int32( vm, realTarget );
 		}
 	}
 	return result;
@@ -460,6 +485,7 @@ void mk_destroy_expr( MK_MEMORY_POOL *top, MK_NODE_EXPR *target )
 			break;
 
 		case MK_TYPE_NODE_EXPR_OPERATION:
+		case MK_TYPE_NODE_EXPR_BACK_IF_CONDITION:
 			mk_destroy_node( top, target->u1.left );
 			mk_destroy_node( top, target->u2.right );
 			break;
@@ -497,12 +523,9 @@ void mk_destroy_expr( MK_MEMORY_POOL *top, MK_NODE_EXPR *target )
 			free_if_not_memory_pool( target->u2.value );
 			break;
 
-		case MK_TYPE_NODE_EXPR_INT8:
-		case MK_TYPE_NODE_EXPR_INT16:
 		case MK_TYPE_NODE_EXPR_INT32:
 		case MK_TYPE_NODE_EXPR_INT64:
 		case MK_TYPE_NODE_EXPR_FLOAT:
-		case MK_TYPE_NODE_EXPR_CHAR:
 			break;
 		
 		case MK_TYPE_NODE_EXPR_ARRAY_DEFINITION:
@@ -627,7 +650,10 @@ void mk_destroy_node( MK_MEMORY_POOL *top, void *target )
 		{
 			MK_CLASS *classTarget = 
 				( MK_CLASS * )target;
+			mk_destroy_vector_node( top, classTarget->usingNames );
+
 			mk_destroy_hashtable_node( top, classTarget->variables, 1 );
+
 			if( classTarget->operatorMethod != NULL )
 			{
 				int index = 0;
@@ -646,14 +672,12 @@ void mk_destroy_node( MK_MEMORY_POOL *top, void *target )
 				( MK_VARIABLE * )target;
 			if( variableTarget->flags & MK_TYPE_ATTRIBUTE_VARIABLE_METHOD )
 			{	// method
-				if( !( variableTarget->flags & MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE ) &&
-					!( variableTarget->flags & MK_TYPE_ATTRIBUTE_VARIABLE_VIRTUAL ) )
-				{
-					mk_destroy_vector_node( top, variableTarget->args );
-					mk_destroy_node( top, variableTarget->impl );
-				}
+				mk_destroy_vector_node( top, variableTarget->args );
+				if( !( variableTarget->flags & MK_TYPE_ATTRIBUTE_VARIABLE_VIRTUAL ) &&
+					!( variableTarget->flags & MK_TYPE_ATTRIBUTE_VARIABLE_METHOD_NATIVE ) )
+					mk_destroy_node( top, variableTarget->entryPoint );
 			}
-			else
+			else if( !( variableTarget->flags & MK_TYPE_ATTRIBUTE_VARIABLE_MODULE ) )
 			{	// value
 				mk_destroy_node( top, variableTarget->defaultValue );
 				free_if_not_memory_pool( variableTarget->name );
