@@ -231,24 +231,27 @@ INT_PTR mk_set_at_vector( MK_VECTOR *ptarget, unsigned int index, INT_PTR newIte
 INT_PTR mk_get_at_vector( MK_VECTOR *ptarget, unsigned int index )
 {
 	INT_PTR result = 0;
-	switch( MK_TYPE_ATTRIBUTE_VECTOR_STYLE(ptarget->flags) )
+	if( index < ptarget->used )
 	{
-	case MK_TYPE_VECTOR_ARRAY_MANAGED_PTR:
-	case MK_TYPE_VECTOR_ARRAY_UNMANAGED_PTR:
-		result = ptarget->items[index];
-		break;
-	case MK_TYPE_VECTOR_ARRAY_INT8:
-		result= ptarget->items8[index];
-		break;
-	case MK_TYPE_VECTOR_ARRAY_INT16:
-		result = ptarget->items16[index];
-		break;
-	case MK_TYPE_VECTOR_ARRAY_INT32:
-		result = ptarget->items32[index];
-		break;
-	case MK_TYPE_VECTOR_ARRAY_INT64:
-		result = (INT_PTR)ptarget->items64[index];
-		break;
+		switch( MK_TYPE_ATTRIBUTE_VECTOR_STYLE(ptarget->flags) )
+		{
+		case MK_TYPE_VECTOR_ARRAY_MANAGED_PTR:
+		case MK_TYPE_VECTOR_ARRAY_UNMANAGED_PTR:
+			result = ptarget->items[index];
+			break;
+		case MK_TYPE_VECTOR_ARRAY_INT8:
+			result= ptarget->items8[index];
+			break;
+		case MK_TYPE_VECTOR_ARRAY_INT16:
+			result = ptarget->items16[index];
+			break;
+		case MK_TYPE_VECTOR_ARRAY_INT32:
+			result = ptarget->items32[index];
+			break;
+		case MK_TYPE_VECTOR_ARRAY_INT64:
+			result = (INT_PTR)ptarget->items64[index];
+			break;
+		}
 	}
 	return result;
 }
